
Button = Core.class(Bitmap)

function Button:init(upTexture, options)
  local props = {
    x               = 0,
    y               = 0,
    handleByCenter  = true,
    scale           = 1,
    alpha           = 1,
    enabled         = true,
    onPressed       = nil,
    onReleased      = nil,
  }
  if options then
    for key, value in pairs(options) do
      props[key]= value
    end
  end
  
  self.props = props
  
  self.upTexture = upTexture

  self:setPosition(props.x, props.y)
  self:setAlpha(props.alpha)
  self:setScale(props.scale)
  if props.handleByCenter then
    self:setAnchorPoint(0.5, 0.5)
  end
  
  self.enabled = props.enabled
  self.focus = false
  self.id = -1
  self.held = false

  self:addEventListener(Event.TOUCHES_BEGIN, self.pressed, self)
  self:addEventListener(Event.TOUCHES_END, self.released, self)
end

function Button:pressed(e) -- Handle touch events
  if not self.enabled then return end
  if self:hitTestPoint(e.touch.x, e.touch.y) then
    e:stopPropagation()
    self.id = e.touch.id
    self.focus = true
	self.held = true
    local props = self.props
    if props.onPressed then
      props.onPressed(self) -- Perform action
    end
  end
end

function Button:released(e) -- Handle touch end events
  if not self.enabled then return end
  if e.touch.id == self.id and self.focus and self:hitTestPoint(e.touch.x, e.touch.y) then
    local props = self.props
    
    e:stopPropagation()
    self.focus = false
	self.held = false
    self.id = -1
    if props.onReleased then
      props.onReleased(self) -- Perform action
    end
  end
end
